@@ -1,19 +1,25 @@
 from __future__ import annotations
 from ehelply_bootstrapper.integrations.integration import Integration
 from ehelply_microservice_library.integrations.fact import get_fact_endpoint
+from ehelply_microservice_library.integrations.logging import Log
 
 from ehelply_bootstrapper.utils.state import State
 
-from ehelply_microservice_library.integrations.sdk import SDK
-from ehelply_python_sdk.services.access.auth_rules import AuthException
-
-from ehelply_python_sdk.services.monitor.sdk import *
-
 from pydantic import BaseModel
 from fastapi.encoders import jsonable_encoder
 import json
 
 
+class UsageType(BaseModel):
+    """
+    Usage_type_key: The string usage type key name
+    Scaling: The default amount of scaling to apply to a particular usage type. This will help remove the constant
+        need for *1000000 for usage types that dont need that level of precision
+    """
+    usage_type_key: str
+    scaling: int = 1
+
+
 class UsageMQ(BaseModel):
     project_uuid: str
     usage_key: str
@@ -25,43 +31,17 @@ class Monitor(Integration):
     Monitor integration is used to talk to the ehelply-meta microservice
     """
 
-    def __init__(self) -> None:
+    def __init__(self, supported_usage_types: list[UsageType]) -> None:
         super().__init__("monitor")
 
-        class IntegratedSDK(MonitorSDK):
-            def get_base_url(self) -> str:
-                return get_fact_endpoint('ehelply-monitors')
-
-        self.sdk: IntegratedSDK = IntegratedSDK(
-            sdk_configuration=SDK.instance.sdk_configuration,
-            requests_session=SDK.instance.requests_session
-        )
-
         self.sqs = State.aws.make_client("sqs")
+        self.supported_usage_types: dict[str, int] = {
+            item.usage_type_key: item.scaling for item in supported_usage_types
+        }
 
     def load(self):
         pass
 
-    async def get_project(self, project_uuid: str) -> Optional[GetProjectResponse]:
-        """
-        Automation helper method for returning whether a project spend is maxed as a bool
-
-        Args:
-            project_uuid:
-
-        Returns:
-
-        """
-        if project_uuid == 'ehelply-resources' or project_uuid == 'ehelply-cloud':
-            return None
-
-        get_project_response: GetProjectResponse = await self.sdk.get_project(project_uuid=project_uuid)
-
-        # if is_response_error(get_project_response):
-        #     raise AuthException("Retrieving project details failed.")
-
-        return get_project_response
-
     def add_usage(self, usage: UsageMQ) -> bool:
         """
         Add usage to a project
@@ -70,6 +50,13 @@ class Monitor(Integration):
 
         :return:
         """
+        if usage.usage_key not in self.supported_usage_types:
+            State.logger.warning(
+                f"Usage type has not been registered. Override get_usage_types() in service.py and add {usage.usage_key} there."
+            )
+        else:
+            usage.quantity *= self.supported_usage_types[usage.usage_key]
+
         if usage.project_uuid == 'ehelply-resources' or usage.project_uuid == 'ehelply-cloud':
             return False
 
