@@ -23,6 +23,12 @@ This imports ALL of the integration dependency injection functions. Examples of
   endpoints below
 """
 from ehelply_microservice_library.integrations.router_integrations import *
+from ehelply_microservice_library.integrations.ueri import Ueri, UeriModel
+from ehelply_microservice_library.utils.ehelply_cloud import (
+    ehelply_cloud_access,
+    CloudParticipantAuthRequest,
+    CloudParticipantResponse,
+)
 
 """
 Router
@@ -116,3 +122,20 @@ async def service_seed(integrations: Integrations = Depends(get_integrations), d
         return http_200_ok(response)
 
     return http_500_error(error_message="Seeding failed")
+
+
+@router.get(
+    "/ueri/{ueri}",
+    tags=["service"]
+)
+async def get_ueri(
+        ueri: str,
+        auth: CloudParticipantAuthRequest = Depends(get_auth),
+        ueri_integration: Ueri = Depends(get_ueri)
+):
+    cloud_participant: CloudParticipantResponse = await ehelply_cloud_access(
+        auth=auth
+    )
+    ueri_model = ueri_integration.parse_ueri(ueri)
+    handled_ueri = ueri_integration.handle(cloud_participant, ueri_model)
+    return ueri_integration.drill(ueri_model, handled_ueri)
