@@ -101,25 +101,28 @@ class KPIFetcher(AbstractTimerService):
         return State.bootstrapper.get_kpi()
 
     async def proc(self):
-        cutoff = datetime.datetime.utcnow() - datetime.timedelta(hours=6)
+        try:
+            cutoff = datetime.datetime.utcnow() - datetime.timedelta(hours=6)
 
-        time_0200am = datetime.time(2, 0)
-        time_0400am = datetime.time(4, 0)
+            time_0200am = datetime.time(2, 0)
+            time_0400am = datetime.time(4, 0)
 
-        if KPIFetcher.kpi_holder is None or (
-                KPIFetcher.kpi_holder.fetched_at <= cutoff and self.is_time_between(time_0200am, time_0400am)):
-            self.logger.info("Fetched a new set of KPIs")
-            KPIFetcher.kpi_holder = KPIHolder(kpis=self.fetch_kpis(), fetched_at=datetime.datetime.utcnow())
+            if KPIFetcher.kpi_holder is None or (
+                    KPIFetcher.kpi_holder.fetched_at <= cutoff and self.is_time_between(time_0200am, time_0400am)):
+                self.logger.info("Fetched a new set of KPIs")
+                KPIFetcher.kpi_holder = KPIHolder(kpis=self.fetch_kpis(), fetched_at=datetime.datetime.utcnow())
 
-            kpi_mq: KPIMQ = KPIMQ(
-                kpi_holder=KPIFetcher.kpi_holder,
-                service=self.service_meta
-            )
+                kpi_mq: KPIMQ = KPIMQ(
+                    kpi_holder=KPIFetcher.kpi_holder,
+                    service=self.service_meta
+                )
 
-            self.sqs.send_message(
-                QueueUrl=self.queue_url,
-                MessageBody=json.dumps(jsonable_encoder(kpi_mq.dict()))
-            )
+                self.sqs.send_message(
+                    QueueUrl=self.queue_url,
+                    MessageBody=json.dumps(jsonable_encoder(kpi_mq.dict()))
+                )
+        except Exception as e:
+            self.logger.severe(str(e))
 
 
 class KPIFetcherThread(threading.Thread):
@@ -200,14 +203,17 @@ class MonitorHeartbeat(AbstractBatchingService):
     # logger: Logger = None
 
     def release_batch(self) -> bool:
-        for message in self.get():
-            self.sqs.send_message(
-                QueueUrl=self.queue_url,
-                MessageBody=json.dumps(jsonable_encoder(message))
-            )
+        try:
+            for message in self.get():
+                self.sqs.send_message(
+                    QueueUrl=self.queue_url,
+                    MessageBody=json.dumps(jsonable_encoder(message))
+                )
 
-        if self.uptime_heartbeat_url:
-            requests.post(self.uptime_heartbeat_url)
+            if self.uptime_heartbeat_url:
+                requests.post(self.uptime_heartbeat_url)
+        except Exception as e:
+            self.logger.severe(str(e))
 
 #         try:
 #             uptime_heartbeat_url: str = State.config.better_uptime.heartbeat.url
@@ -224,22 +230,26 @@ class MonitorHeartbeat(AbstractBatchingService):
         return True
 
     def receive(self, limit: int) -> list:
-        service_status_copy: ServiceStatus = ServiceStatus(**self.service_status.dict())
-        self.service_status.health = None
-        self.service_status.vitals = None
-        self.service_status.stats = None
+        try:
+            service_status_copy: ServiceStatus = ServiceStatus(**self.service_status.dict())
+            self.service_status.health = None
+            self.service_status.vitals = None
+            self.service_status.stats = None
 
-        heartbeat: Heartbeat = Heartbeat(
-            service=self.service_meta,
-            platform=self.service_platform,
-            status=service_status_copy,
-            process=self.logger.prefix,
-            stage=self.environment,
-        )
+            heartbeat: Heartbeat = Heartbeat(
+                service=self.service_meta,
+                platform=self.service_platform,
+                status=service_status_copy,
+                process=self.logger.prefix,
+                stage=self.environment,
+            )
 
-        time.sleep(self.sleep_interval)
+            time.sleep(self.sleep_interval)
 
-        return [heartbeat]
+            return [heartbeat]
+        except Exception as e:
+            self.logger.severe(str(e))
+            return []
 
     def is_message_valid(self, message) -> bool:
         return True
