@@ -4,6 +4,9 @@ Not much interesting here. We have some default imports for FastAPI endpoints an
 """
 from fastapi import APIRouter, Body
 
+from ehelply_microservice_library.integrations.fact import get_fact_endpoint
+from ehelply_microservice_library.integrations.sdk import SDK
+
 """
 eHelply Bootstrapper
 Now we are getting interesting.
@@ -21,6 +24,7 @@ from ehelply_batcher.abstract_timer_service import AbstractTimerService
 from ehelply_logger.Logger import Logger
 import threading
 import asyncio
+import requests
 
 """
 Router
@@ -58,20 +62,31 @@ async def remove_key(category_name: str, data: dict = Body({})):
     return http_200_ok({"message": "Removed key"})
 
 
+def get_encryption_keys(category: str, secret_key: str):
+    return requests.get(
+        get_fact_endpoint("ehelply-security") + f"/encryption/categories/{category}/keys",
+        headers={'EHELPLY-SECURITY-SECRET-KEY': secret_key}
+    ).json()
+
+
+def create_encryption_key(category: str, secret_key: str):
+    return requests.post(
+        get_fact_endpoint("ehelply-security") + f"/encryption/categories/{category}/keys",
+        headers={'EHELPLY-SECURITY-SECRET-KEY': secret_key}
+    ).json()
+
+
 async def refresh_encryption_keys(category: str):
     from ehelply_bootstrapper.utils.db_encryption import DBEncryption
-    from ehelply_microservice_library.integrations.security import Security
     from ehelply_microservice_library.utils.secret_names import secret_name_security_vault
 
-    security_integration: Security = State.integrations.get('security')
-
     secret_key: str = State.secrets.get(secret_name_security_vault())[0].secret_key
 
-    keys: list = await security_integration.get_encryption_keys(category=category, secret_key=secret_key)
+    keys: list = get_encryption_keys(category=category, secret_key=secret_key)
 
     if not keys:
-        await security_integration.create_encryption_key(category=category, secret_key=secret_key)
-        keys = await security_integration.get_encryption_keys(category=category, secret_key=secret_key)
+        create_encryption_key(category=category, secret_key=secret_key)
+        keys = get_encryption_keys(category=category, secret_key=secret_key)
 
     for key in keys:
         if not State.secrets.exists(secret=key['key'], category='database'):
@@ -94,7 +109,11 @@ class SecurityKeys(AbstractTimerService):
         )
 
     async def proc(self):
-        await refresh_encryption_keys(category=self.category)
+        try:
+            await refresh_encryption_keys(category=self.category)
+        except Exception as e:
+            self.logger.severe(str(e))
+
 
 
 class SecurityKeysThread(threading.Thread):
