@@ -1,4 +1,4 @@
-from typing import List, Union, Tuple
+from typing import List, Union, Tuple, Callable
 from pathlib import Path
 import glob
 import os
@@ -6,6 +6,7 @@ import json
 import asyncio
 
 from slugify import slugify
+from pydantic import BaseModel
 
 from ehelply_bootstrapper.bootstrap import Bootstrap, LOADABLE_MYSQL
 from ehelply_bootstrapper.drivers.mysql import MySQLCredentials
@@ -21,27 +22,20 @@ from ehelply_microservice_library.integrations.m2m import M2M
 from ehelply_microservice_library.integrations.sdk import SDK
 from ehelply_microservice_library.integrations.fact import Fact, FactBase, get_fact_endpoint, get_fact_stage
 from ehelply_microservice_library.integrations.logging import Log
-from ehelply_microservice_library.integrations.meta import Meta
-from ehelply_microservice_library.integrations.note import Note
-from ehelply_microservice_library.integrations.monitor import Monitor
-from ehelply_microservice_library.integrations.user import User
-from ehelply_microservice_library.integrations.access import Access
-from ehelply_microservice_library.integrations.security import Security
-from ehelply_microservice_library.integrations.appointments import Appointments
-from ehelply_microservice_library.integrations.places import Places
-from ehelply_microservice_library.integrations.reviews import Reviews
-from ehelply_microservice_library.integrations.products import Products
+from ehelply_microservice_library.integrations.monitor import Monitor, UsageType
+from ehelply_microservice_library.integrations.ueri import Ueri
 from ehelply_microservice_library.integrations.notifications import Notifications, NotificationType
 
 from ehelply_microservice_library.utils.constants.constants_gatekeeper import *
 from ehelply_microservice_library.utils.service_updater import ServiceUpdater
 from ehelply_microservice_library.utils.secret_names import secret_name_database, secret_name_facts_vault, \
-    secret_name_security_vault, secret_name_updater_vault
+    secret_name_security_vault, secret_name_updater_vault, secret_name_auth_dynamo
 
 from ehelply_microservice_library.utils.ehelply_cloud import AuthDynamos
 
 CONST_INTEGRATION_ACCESS: str = "access"
 CONST_INTEGRATION_APPOINTMENTS: str = "appointments"
+CONST_INTEGRATION_CONTENT: str = "content"
 CONST_INTEGRATION_META: str = "meta"
 CONST_INTEGRATION_NOTES: str = "notes"
 CONST_INTEGRATION_NOTIFICATIONS: str = "notifications"
@@ -338,6 +332,15 @@ class ServiceBootstrap(Bootstrap):
         """
         return []
 
+    def get_usage_types(self) -> list[UsageType]:
+        """
+        List of up to date UsageTypes
+
+        Returns:
+            list[UsageType]: UsageType's supported by this service
+        """
+        return []
+
     def is_load_service_routers(self) -> bool:
         """
         Whether or not to register the service routers or not
@@ -346,6 +349,12 @@ class ServiceBootstrap(Bootstrap):
         """
         return True
 
+    def get_ueri_handlers(self) -> dict[str, Callable[[str], BaseModel | dict | list]]:
+        return {}
+
+    def get_ueri_custom_drill(self) -> Callable[[list[str], BaseModel | dict | list], any] | None:
+        return None
+
     def fastapi_routers(self):
         """
         Register fastapi routers
@@ -412,7 +421,8 @@ class ServiceBootstrap(Bootstrap):
         subscribe_to_secrets: list = [
             secret_name_facts_vault(),
             secret_name_security_vault(),
-            secret_name_updater_vault()
+            secret_name_updater_vault(),
+            secret_name_auth_dynamo()
         ]
 
         if LOADABLE_MYSQL in self.get_service_loadables():
@@ -463,6 +473,9 @@ class ServiceBootstrap(Bootstrap):
         if CONST_INTEGRATION_META in active_integrations:
             subscribe_to_facts.append("ehelply-meta")
 
+        if CONST_INTEGRATION_CONTENT in active_integrations:
+            subscribe_to_facts.append("ehelply-content")
+
         if CONST_INTEGRATION_USERS in active_integrations:
             subscribe_to_facts.append("ehelply-users")
 
@@ -496,39 +509,13 @@ class ServiceBootstrap(Bootstrap):
 
         # Register the core service integration
         self.register_integration(Log(self.service_meta.key))
-        self.register_integration(Monitor())
+        self.register_integration(Monitor(self.get_usage_types()))
+        self.register_integration(Ueri(self.service_meta, self.get_ueri_handlers(), self.get_ueri_custom_drill()))
 
         # Register the service integrations
-        if CONST_INTEGRATION_ACCESS in active_integrations:
-            self.register_integration(Access())
-
-        if CONST_INTEGRATION_SECURITY in active_integrations:
-            self.register_integration(Security())
-
-        if CONST_INTEGRATION_NOTES in active_integrations:
-            self.register_integration(Note())
-
-        if CONST_INTEGRATION_META in active_integrations:
-            self.register_integration(Meta())
-
-        if CONST_INTEGRATION_USERS in active_integrations:
-            self.register_integration(User())
-
         if CONST_INTEGRATION_NOTIFICATIONS in active_integrations:
             self.register_integration(Notifications(self.get_notification_types()))
 
-        if CONST_INTEGRATION_PLACES in active_integrations:
-            self.register_integration(Places())
-
-        if CONST_INTEGRATION_REVIEWS in active_integrations:
-            self.register_integration(Reviews())
-
-        if CONST_INTEGRATION_APPOINTMENTS in active_integrations:
-            self.register_integration(Appointments())
-
-        if CONST_INTEGRATION_PRODUCTS in active_integrations:
-            self.register_integration(Products())
-
     def get_mysql_credentials(self) -> MySQLCredentials:
         """
         Returns MySQL credentials
