@@ -57,3 +57,14 @@ def secret_name_updater_vault():
 
     """
     return "ehelply-all.microservices.ehelply-updater.vault".format(environment=_get_environment_name())
+
+
+def secret_name_auth_dynamo():
+    """
+    AWS ASM secret name for access to auth dynamo encryption
+
+    Returns:
+        None
+
+    """
+    return "ehelply-{environment}.microservices.auth-dynamos.vault".format(environment=_get_environment_name())
